@@ -16,6 +16,7 @@ import string
 import functools
 import uuid
 import redis
+import tornado.gen
 from dhkit import log
 
 
@@ -247,6 +248,37 @@ class Cacheable(object):
             return decorated
         return _cacheable
 
+    def async_cacheable(self, timeout=None):
+        def _async_cacheable(func):
+            func = tornado.gen.coroutine(func)
+
+            @functools.wraps(func)
+            def decorated(*args, **kwargs):
+                try:
+                    cache_key = decorated.make_cache_key(func, *args, **kwargs)
+                    cache_value = self.cache.get(cache_key)
+                except Exception:
+                    log.exception("Exception possibly due to cache backend.")
+                    rv = yield func(*args, **kwargs)
+                    raise tornado.gen.Return(rv)
+
+                if cache_value is None:
+                    rv = yield func(*args, **kwargs)
+                    try:
+                        cache_value = cPickle.dumps(rv)
+                        self.cache.set(cache_key, cache_value, expire_time=decorated.cache_timeout)
+                    except Exception:
+                        log.exception("Exception possibly due to cache backend.")
+                else:
+                    rv = cPickle.loads(cache_value)
+                raise tornado.gen.Return(rv)
+
+            decorated.uncached_func = func
+            decorated.cache_timeout = timeout or self.cache_timeout
+            decorated.make_cache_key = self._make_cache_key()
+            return decorated
+        return _async_cacheable
+
     def delete_cache(self, func, *args, **kwargs):
         try:
             if not args and not kwargs:
