@@ -11,12 +11,15 @@ Copyright (c) 2014 infohold inc. All rights reserved.
 import json
 import mimetypes
 import itertools
+import base64
+import datetime
+import urllib
 from dhkit import log
 from tornado.httpclient import HTTPClient, HTTPError, HTTPRequest
 from dhkit.error import Error
 from mimetools import choose_boundary
 from dhkit.util import gzip_compress, urlconcat, urlencode
-from oauthlib.oauth2 import LegacyApplicationClient, OAuth2Error
+from oauthlib.oauth2 import LegacyApplicationClient, BackendApplicationClient, OAuth2Error
 
 
 class ApiError(Error):
@@ -68,16 +71,24 @@ class ShouYiApi(Api):
 
     API_URL = "http://api.ishouyi.com"
 
-    def __init__(self, api_url=None, timeout=30, use_gzip=True):
+    __access_token = None
+    __last_access_time = None
+    __expire_in = None
+
+    def __init__(self, api_url=None, timeout=30, use_gzip=True, api_authorized=False, client_id=None, client_secret=None):
         self.api_url = api_url or self.API_URL
         self.timeout = timeout
         self.use_gzip = use_gzip
+        self.api_authorized = api_authorized
+        self.client_id = client_id
+        self.client_secret = client_secret
 
     def get(self, api_path, **kwargs):
         headers = {
             'Accept': "application/json",
             'Connection': 'Keep-Alive',
         }
+        self.set_access_token(headers)
 
         if kwargs:
             url = urlconcat(self.part_url(api_path), kwargs)
@@ -93,6 +104,7 @@ class ShouYiApi(Api):
             'Accept': "application/json",
             'Connection': 'Keep-Alive',
         }
+        self.set_access_token(headers)
         url = self.part_url(api_path)
         body = kwargs and urlencode(kwargs) or None
         request = HTTPRequest(url, method="POST", headers=headers, body=body, connect_timeout=self.timeout,
@@ -129,7 +141,34 @@ class ShouYiApi(Api):
         api_url = self.api_url[:-1] if self.api_url.endswith("/") else self.api_url
         return "%s%s" % (api_url, api_path)
 
-    def oauth(self, client_id, client_secret, username, password):
+    @property
+    def access_token(self):
+        now = datetime.datetime.now()
+        if self.__access_token is None or self.__last_access_time + datetime.timedelta(seconds=self.__expire_in) >= now:
+            token_dict = self.get_token(self.client_id, self.client_secret)
+            self.__access_token = token_dict.get('access_token')
+            self.__expire_in = token_dict.get('expires_in')
+            self.__last_access_time = now
+        return self.__access_token
+
+    def set_access_token(self, headers):
+        if not self.api_authorized:
+            return
+        headers['Authorization'] = 'Bearer %s' % self.access_token
+
+    def get_token(self, client_id, client_secret):
+        authorization = '%s:%s' % (urllib.quote(client_id), urllib.quote(client_secret))
+        headers = {
+            'Authorization': 'Basic %s' % base64.b64encode(authorization),
+            'Content-Type': 'application/x-www-form-urlencoded',
+        }
+        body = 'grant_type=client_credentials'
+        url = self.part_url('/oauth/token')
+        request = HTTPRequest(url, method="POST", headers=headers, body=body, connect_timeout=self.timeout,
+                              request_timeout=self.timeout, use_gzip=self.use_gzip, validate_cert=False)
+        return self.fetch(request)
+
+    def authorize(self, client_id, client_secret, username, password):
         """OAUTH2认证，基于Username and Password Flow
 
         认证成功之后将会返回一个字典，字典里属性说明如下：
